# %%
import json
from llmlinks.function import LLMFunction
from llmlinks.llm_client import LLMClient


class LLMComponent:
    def __init__(self, llm_name, json_file_path=None, json_data=None):
        self.llm_name = llm_name
        if json_file_path:
            with open(json_file_path, 'r') as file:
                self.json_data = json.load(file)
        elif json_data:
            if isinstance(json_data, str):
                self.json_data = json.loads(json_data)
            elif isinstance(json_data, dict):
                self.json_data = json_data
        else:
            raise ValueError("Either json_file_path or json_data must be provided.")
        self.input = self.json_data.get('input')
        self.output = self.json_data.get('output')
        self.prompt_template = self.json_data.get('prompt')
        print("LLMComponent initialized")
        print(f"input: {self.input}")
        print(f"output: {self.output}")

    def __call__(self, memory_):
        """LLMComponentの実行

        Args:
            memory_ (_type_): _description_

        Returns:
            _type_: _description_
        """
        llm = LLMClient(self.llm_name)
        
        # LLMを複数回実行する場合
        if isinstance(self.input[0], list):
            num_loop = len(self.input)
            for i in range(num_loop):
                prompt = self.prompt_template[i]
                func = LLMFunction(
                    llm, 
                    prompt,
                    self.input[i],
                    self.output[i],
                )
                
                kwargs = {key: memory_[key] for key in self.input[i]}
                response = func(**kwargs)
                print(response)
                for key in self.output[i]:
                    memory_[key] = response[key][0]
            
        # LLMを一回だけ実行する場合
        else:
            func = LLMFunction(
                llm, 
                self.prompt_template,
                self.input,
                self.output,
            )
            
            kwargs = {key: memory_[key] for key in self.input}
            response = func(**kwargs)
            for key in self.output:
                memory_[key] = response[key][0]
        return memory_


if __name__ == "__main__":
    memory = {
        'source': 'Hello World!!',
        'language': 'japanese',
        'output': None
        }
        
    # 基本となる処理の実行
    llm_name = 'gpt-4o-2024-08-06'
    json_file_path = './test/base.json'
    translate = LLMComponent(llm_name=llm_name, json_file_path=json_file_path)
    memory1 = translate(memory.copy())
    print(memory1)
    
    # outputが二つある場合の実行
    llm_name = 'gpt-4o-2024-08-06'
    json_file_path = './test/two_outputs.json'
    translate2 = LLMComponent(llm_name=llm_name, json_file_path=json_file_path)
    memory2 = translate2(memory.copy())
    print(memory2)
    
    # llmの実行が2回ある場合の実行
    llm_name = 'gpt-4o-2024-08-06'
    json_file_path = './test/two_runs.json'
    translate3 = LLMComponent(llm_name=llm_name, json_file_path=json_file_path)
    memory3 = translate3(memory.copy())
    print(memory3)

    # jsonデータを直接渡す場合
    llm_name = 'gpt-4o-2024-08-06'
    json_data = {
        "input_variables": ["source", "language"],
        "output_variables": ["output"],
        "prompt_template": "<source_text>\n{source}\n</source_text>\n<target_language>\n{language}\n</target_language>\n<rule>\nsource_text タグで与えられた文章を target_language で指定された言語に翻訳して output タグを用いて出力せよ．\n</rule>"
    }
    translate4 = LLMComponent(llm_name=llm_name, json_data=json_data)
    memory4 = translate4(memory.copy())
    print(memory4)



# %%
from llm_component.llm_component import LLMComponent
from component.retriever.semantic_scholar import SemanticScholarRetriever
from component.retriever.github import GithubRetriever

if __name__ == "__main__":
    llm_name = 'gemini-1.5-pro'
    save_dir = "/workspaces/researchchain/data"

    retriever = SemanticScholarRetriever(save_dir=save_dir)
    keyworder = LLMComponent(
        llm_name=llm_name,
        json_file_path='llm_component/hotl/keyworder/keyworder.json'
        )
    selector = LLMComponent(
        llm_name=llm_name,
        json_file_path='./llm_component/hotl/selector/selector1.json'
        )
    extractor = LLMComponent(
        llm_name=llm_name,
        json_file_path='./llm_component/hotl/extractor/extractor_pre_method.json'
        )
    githubretriever = GithubRetriever(save_dir=save_dir)
    codeextractor = LLMComponent(
        llm_name=llm_name,
        json_file_path='./llm_component/hotl/codeextractor/codeextractor_patch_method.json'
        )
    decomposer = LLMComponent(
        llm_name=llm_name,
        json_file_path='./llm_component/hotl/decomposer/decomposer_patch_method.json'
        )

    memory = {
        'environment' : 'The following two experimental environments are available・Fine tuning of the LLM and experiments with rewriting the Optimizer or loss function.・Verification of the accuracy of prompt engineering.',
        'objective' : 'Combining two papers to create a new methodology',
        'keywords' : 'LLM'
    }

    #memory = keyworder(memory)
    #print(memory)
    memory = retriever(memory)
    print(memory)
    #memory = selector(memory)
    #memory = extractor(memory)
    #memory = githubretriever(memory)
    #memory = codeextractor(memory)
    #memory = decomposer(memory)




import os
import subprocess
import glob

class GithubRetriever:
    def __init__(self, save_dir):
        self.save_dir = save_dir
        print("GithubRetriever initialized")
        print("input: ['github_url']")
        print("output: ['folder_structure', 'github_file']")
    
    def get_folder_structure(self, path='.'):
        try:
            # subprocess.runを使用してフォルダ構造を取得
            result = subprocess.run(['ls', '-R', path], capture_output=True, text=True, check=True)
            return result.stdout
        except subprocess.CalledProcessError as e:
            print(f"Error occurred: {e}")
            return None

    def git_clone(self, url):
        if not os.path.exists(self.save_dir):
            os.makedirs(self.save_dir)
        os.chdir(self.save_dir)
        command = f'git clone {url}'
        ls = subprocess.run(command, shell=True, text=True)
        return ls

    def get_py_files(self, url):
        folder_structure = self.get_folder_structure(url.split('/')[-1])
        py_filelist = glob.glob(url.split('/')[-1]+"/**/*.py", recursive=True)
        py_text = []
        for i in py_filelist:
            with open(i) as f:
                file_read = f.read()
        py_text.append('<FILE='+i+'> \n'+file_read.replace('\n\n', '\n').replace('\n\n', '\n')+'</FILE> \n')
        result = ''.join(py_text)
        return folder_structure,result

    def __call__(self, memory):
        github_url = memory["github_url"]
        self.git_clone(github_url)
        folder_structure,get_file = self.get_py_files(github_url)
        memory["folder_structure"] = folder_structure
        memory["github_file"] = get_file
        return memory


if __name__ == "__main__":
    save_dir = "/workspaces/researchchain/data"
    memory = {
        "github_url": "https://github.com/fuyu-quant/IBLM"
    }
    githubretriever = GithubRetriever(save_dir=save_dir)
    memory = githubretriever(memory)
    print(memory)



# %%
import os
import shutil
import requests
from langchain_community.document_loaders import PyPDFLoader
from semanticscholar import SemanticScholar

class SemanticScholarRetriever:
    def __init__(self, save_dir):
        self.save_dir = save_dir
        print("SemanticScholarRetriever initialized")
        print("input: ['keywords']")
        print("output: ['collection_of_papers_1']")

    def download_from_arxiv_id(self, arxiv_id, save_dir):
        """Download PDF file from arXiv

        Args:
            arxiv_id (_type_): _description_
            save_dir (_type_): _description_
        """
        
        url = f"https://arxiv.org/pdf/{arxiv_id}.pdf"
        response = requests.get(url, stream=True)

        if response.status_code == 200:
            with open(os.path.join(save_dir, f"{arxiv_id}.pdf"), 'wb') as file:
                shutil.copyfileobj(response.raw, file)
            print(f"Downloaded {arxiv_id}.pdf to {save_dir}")
        else:
            print(f"Failed to download {arxiv_id}.pdf")

    def download_from_arxiv_ids(self, arxiv_ids, save_dir):
        """Download PDF files from arXiv

        Args:
            arxiv_ids (_type_): _description_
            save_dir (_type_): _description_
        """
        # save_dirが存在しない場合、ディレクトリを作成
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        else :
            shutil.rmtree(save_dir)
            os.makedirs(save_dir)

        for arxiv_id in arxiv_ids:
            self.download_from_arxiv_id(arxiv_id, save_dir)

    def convert_pdf_to_text(self, pdf_path):
        """Convert PDF file to text

        Args:
            pdf_path (_type_): _description_

        Returns:
            _type_: _description_
        """

        loader = PyPDFLoader(pdf_path)
        pages = loader.load_and_split()
        content = ""
        for page in pages[:20]:
            content += page.page_content

        return content

    def __call__(self, memory):
        """Retriever

        Args:
            memory (_type_): _description_
        """
        sch = SemanticScholar()
        results = sch.search_paper(memory["keywords"][0], limit=10)
        for item in results.items:
            print(item.title)
            print(item.paperId)

        DOI_ids = [item['externalIds'] for item in results.items]
        arxiv_ids = [item['ArXiv'] for item in DOI_ids if 'ArXiv' in item]

        self.download_from_arxiv_ids(arxiv_ids[:3], self.save_dir)

        # ディレクトリ内のすべてのPDFファイルを処理
        for idx, filename in enumerate(os.listdir(self.save_dir)):
            if filename.endswith('.pdf'):
                pdf_path = os.path.join(self.save_dir, filename)
                paper_content = self.convert_pdf_to_text(pdf_path)
                paper_key = f"paper_1_{idx+1}"
                memory["collection_of_papers_1"][paper_key] = paper_content
    
        return memory
    
    
if __name__ == "__main__":
    save_dir = "/workspaces/researchchain/data"
    memory = {
        "keywords": ["llm", "optimizer", "loss function"],
        "collection_of_papers_1": {}
    }
    retriever = SemanticScholarRetriever(save_dir=save_dir)
    memory = retriever(memory)
    print(memory)
